@@ -28,11 +28,14 @@ from twisted.internet.defer import Deferred
 
 from log import m18nE, logInfo, logDebug, logException
 from message import Message
-from common import Debug
+from common import Debug, isPython3, StrMixin, unicode, nativeString, nativeStringArgs
 from move import Move
 
-class Request(object):
+
+class Request(StrMixin):
+
     """holds a Deferred and related attributes, used as part of a DeferredBlock"""
+
     def __init__(self, block, deferred, user, about):
         self._block = weakref.ref(block)
         self.deferred = deferred
@@ -71,43 +74,40 @@ class Request(object):
     def gotAnswer(self, rawAnswer):
         """convert the wired answer into something more useful"""
         if isinstance(rawAnswer, tuple):
-            answer = rawAnswer[0]
+            answer = nativeString(rawAnswer[0])
             if isinstance(rawAnswer[1], tuple):
-                self.args = list(rawAnswer[1])
+                self.args = nativeStringArgs(rawAnswer[1])
             else:
-                self.args = list([rawAnswer[1]])
+                self.args = nativeStringArgs([rawAnswer[1]])
         else:
-            answer = rawAnswer
+            answer = nativeString(rawAnswer)
             self.args = None
         if answer in Message.defined:
             self.answer = Message.defined[answer]
         else:
             if Debug.deferredBlock:
-                logDebug('Request %s ignores %s' % (self, rawAnswer))
+                logDebug(u'Request %s ignores %s' % (self, rawAnswer))
 
     def age(self):
         """my age in full seconds"""
         return int((datetime.datetime.now() - self.startTime).total_seconds())
 
-    def __str__(self):
+    def __unicode__(self):
         cmd = self.deferred.command
         if self.answer:
-            answer = str(self.answer)
+            answer = unicode(self.answer)
         else:
-            answer = 'OPEN'
-        result = ''
+            answer = u'OPEN'
+        result = u''
         if Debug.deferredBlock:
-            result += '[{id:>4}] '.format(id=id(self)%10000)
-        result += '{cmd}->{cls}({receiver:<10}): {answer}'.format(
+            result += u'[{id:>4}] '.format(id=id(self) % 10000)
+        result += u'{cmd}->{cls}({receiver:<10}): {answer}'.format(
             cls=self.user.__class__.__name__, cmd=cmd, receiver=self.user.name,
             answer=answer)
         if self.age():
-            result += ' after {} sec'.format(self.age())
+            result += u' after {} sec'.format(self.age())
         return result
 
-    def __repr__(self):
-        return 'Request(%s)' % str(self)
-
     def prettyAnswer(self):
         """for debug output"""
         if self.answer:
@@ -120,24 +120,26 @@ class Request(object):
 
     def pretty(self):
         """for debug output"""
-        result = ''
+        result = u''
         if Debug.deferredBlock:
-            result += '[{id:>4}] '.format(id=id(self)%10000)
-        result += '{cmd:<12}<-{cls:>6}({receiver:<10}): ANS={answer}'.format(
+            result += u'[{id:>4}] '.format(id=id(self) % 10000)
+        result += u'{cmd:<12}<-{cls:>6}({receiver:<10}): ANS={answer}'.format(
             cls=self.user.__class__.__name__,
             answer=self.prettyAnswer(), cmd=self.deferred.command, receiver=self.user.name)
         if self.age() > 0:
-            result += ' after {} sec'.format(self.age())
+            result += u' after {} sec'.format(self.age())
         return result
 
-class DeferredBlock(object):
+
+class DeferredBlock(StrMixin):
+
     """holds a list of deferreds and waits for each of them individually,
     with each deferred having its own independent callbacks. Fires a
     'general' callback after all deferreds have returned.
     Usage: 1. define, 2. add requests, 3. set callback"""
 
     blocks = []
-    blockWarned = False # did we already warn about too many blocks?
+    blockWarned = False  # did we already warn about too many blocks?
 
     def __init__(self, table, temp=False):
         dummy, dummy, function, dummy = traceback.extract_stack()[-2]
@@ -159,28 +161,28 @@ class DeferredBlock(object):
                     for block in DeferredBlock.blocks:
                         logInfo(str(block))
 
-    def debugPrefix(self, marker=''):
+    def debugPrefix(self, marker=u''):
         """prefix for debug message"""
-        return 'T{table} B[{id:>4}] {caller:<15} {marker:<3}(out={out})'.format(
+        return u'T{table} B[{id:>4}] {caller:<15} {marker:<3}(out={out})'.format(
             table=self.table.tableid, id=id(self) % 10000, caller=self.calledBy[:15],
             marker=marker, out=self.outstanding)
 
     def debug(self, marker, msg):
         """standard debug format"""
-        logDebug(' '.join([self.debugPrefix(marker), msg]))
+        logDebug(u' '.join([self.debugPrefix(marker), msg]))
 
-    def __str__(self):
-        return '%s requests=%s outstanding=%d %s callback=%s' % (
+    def __unicode__(self):
+        return u'%s requests=%s outstanding=%d %s callback=%s' % (
             self.debugPrefix(),
-            '[' + ','.join(str(x) for x in self.requests) + ']',
+            u'[' + u','.join(unicode(x) for x in self.requests) + u']',
             self.outstanding,
-            'is completed' if self.completed else 'not completed',
+            u'is completed' if self.completed else u'not completed',
             self.prettyCallback())
 
-    def outstandingStr(self):
-        """like __str__ but only with outstanding answers"""
-        return '%s callback=%s:%s' % (self.calledBy, self.prettyCallback(),
-            '[' + ','.join(str(x) for x in self.requests if not x.answer) + ']')
+    def outstandingUnicode(self):
+        """like __unicode__ but only with outstanding answers"""
+        return u'%s callback=%s:%s' % (self.calledBy, self.prettyCallback(),
+                                       u'[' + u','.join(str(x) for x in self.requests if not x.answer) + u']')
 
     @staticmethod
     def garbageCollection():
@@ -189,11 +191,13 @@ class DeferredBlock(object):
         never overlaps."""
         for block in DeferredBlock.blocks[:]:
             if block.callbackMethod is None:
-                block.logBug('DBlock %s has no callback' % str(block))
+                block.logBug(u'DBlock %s has no callback' % str(block))
             if block.completed:
                 DeferredBlock.blocks.remove(block)
         if len(DeferredBlock.blocks) > 100:
-            logDebug('We have %d DeferredBlocks, they must be leaking' % len(DeferredBlock.blocks))
+            logDebug(
+                u'We have %d DeferredBlocks, they must be leaking' %
+                len(DeferredBlock.blocks))
 
     def __addRequest(self, deferred, user, about):
         """add deferred for user to this block"""
@@ -202,12 +206,16 @@ class DeferredBlock(object):
         request = Request(self, deferred, user, about)
         self.requests.append(request)
         self.outstanding += 1
-        deferred.addCallback(self.__gotAnswer, request).addErrback(self.__failed, request)
+        deferred.addCallback(
+            self.__gotAnswer,
+            request).addErrback(
+                self.__failed,
+                request)
         if Debug.deferredBlock:
             notifying = ' notifying' if deferred.notifying else ''
-            rqString = '[{id:>4}] {cmd}{notifying} {about}->{cls:>6}({receiver:<10})'.format(
+            rqString = u'[{id:>4}] {cmd}{notifying} {about}->{cls:>6}({receiver:<10})'.format(
                 cls=user.__class__.__name__,
-                id=id(request)%10000, cmd=deferred.command, receiver=user.name,
+                id=id(request) % 10000, cmd=deferred.command, receiver=user.name,
                 about=about.name if about else '', notifying=notifying)
             self.debug('+:%d' % len(self.requests), rqString)
 
@@ -217,7 +225,7 @@ class DeferredBlock(object):
         if not request.answer:
             self.outstanding -= 1
         if Debug.deferredBlock:
-            self.debug('-:%d' % self.outstanding, str(request))
+            self.debug(u'-:%d' % self.outstanding, unicode(request))
         self.callbackIfDone()
 
     def callback(self, method, *args):
@@ -233,7 +241,8 @@ class DeferredBlock(object):
     def __gotAnswer(self, result, request):
         """got answer from user"""
         if request in self.requests:
-            # after having lost connection to client, an answer could still be in the pipe
+            # after having lost connection to client, an answer could still be
+            # in the pipe
             if result is None:
                 if Debug.deferredBlock:
                     self.debug('IGN', request.pretty())
@@ -249,7 +258,11 @@ class DeferredBlock(object):
                     block = DeferredBlock(self.table, temp=True)
                     receivers = request.answer.receivers(request)
                     if receivers:
-                        block.tell(request.player, receivers, request.answer, notifying=True)
+                        block.tell(
+                            request.player,
+                            receivers,
+                            request.answer,
+                            notifying=True)
             self.outstanding -= 1
             assert self.outstanding >= 0, '__gotAnswer: outstanding %d' % self.outstanding
             self.callbackIfDone()
@@ -270,13 +283,17 @@ class DeferredBlock(object):
                 traceBack = result.getTraceback()
             except BaseException:
                 # may happen with twisted 12.3.0
-                traceBack = 'twisted cannot give us a traceback'
-            self.table.abort(msg, request.user.name, result.getErrorMessage(), traceBack)
+                traceBack = u'twisted cannot give us a traceback'
+            self.table.abort(
+                msg,
+                request.user.name,
+                result.getErrorMessage(),
+                traceBack)
 
     def logBug(self, msg):
         """log msg and raise exception"""
         for request in self.requests:
-            logDebug(str(request))
+            logDebug(unicode(request))
         logException(msg)
 
     def callbackIfDone(self):
@@ -287,43 +304,52 @@ class DeferredBlock(object):
         if self.outstanding == 0 and self.callbackMethod is not None:
             self.completed = True
             if any(not x.answer for x in self.requests):
-                self.logBug('Block %s: Some requests are unanswered' % str(self))
+                self.logBug(
+                    u'Block %s: Some requests are unanswered' %
+                    str(self))
             if Debug.deferredBlock:
                 commandText = []
                 for command in set(x.deferred.command for x in self.requests):
-                    text = '%s:' % command
+                    text = u'%s:' % command
                     answerList = []
                     for answer in set(x.prettyAnswer() for x in self.requests if x.deferred.command == command):
                         answerList.append((answer, list(x for x in self.requests
-                            if x.deferred.command == command and answer == x.prettyAnswer())))
+                                                        if x.deferred.command == command and answer == x.prettyAnswer())))
                     answerList = sorted(answerList, key=lambda x: len(x[1]))
                     answerTexts = []
                     if len(answerList) == 1:
-                        answerTexts.append('{answer} from all'.format(answer=answerList[-1][0]))
+                        answerTexts.append(
+                            u'{answer} from all'.format(answer=answerList[-1][0]))
                     else:
                         for answer, requests in answerList[:-1]:
-                            answerTexts.append('{answer} from {players}'.format(answer=answer,
-                                players=','.join(x.user.name for x in requests)))
-                        answerTexts.append('{answer} from others'.format(answer=answerList[-1][0]))
-                    text += ', '.join(answerTexts)
+                            answerTexts.append(
+                                u'{answer} from {players}'.format(answer=answer,
+                                                                  players=u','.join(x.user.name for x in requests)))
+                        answerTexts.append(
+                            u'{answer} from others'.format(answer=answerList[-1][0]))
+                    text += u', '.join(answerTexts)
                     commandText.append(text)
                 methodName = self.prettyCallback()
                 if methodName:
-                    methodName = ' next:%s' % methodName
-                self.debug('END', '{answers} {method}'.format(method=methodName, answers=' / '.join(commandText)))
+                    methodName = u' next:%s' % methodName
+                self.debug(
+                    u'END',
+                    u'{answers} {method}'.format(method=methodName,
+                                                 answers=u' / '.join(commandText)))
             if self.callbackMethod is not False:
                 self.callbackMethod(self.requests, *self.__callbackArgs)
 
     def prettyCallback(self):
         """pretty string for callbackMethod"""
         if self.callbackMethod is False:
-            result = ''
+            result = u''
         elif self.callbackMethod is None:
-            result = 'None'
+            result = u'None'
         else:
             result = self.callbackMethod.__name__
             if self.__callbackArgs:
-                result += '({})'.format(','.join([str(x) for x in self.__callbackArgs] if self.__callbackArgs else ''))
+                result += u'({})'.format(
+                    u','.join([str(x) for x in self.__callbackArgs] if self.__callbackArgs else u''))
         return result
 
     def playerForUser(self, user):
@@ -343,7 +369,8 @@ class DeferredBlock(object):
             # very much in finding bugs.
             kwargs['score'] = str(about.hand)
         if game and game.gameid and 'token' not in kwargs:
-            # this lets the client assert that the message is meant for the current hand
+            # this lets the client assert that the message is meant for the
+            # current hand
             kwargs['token'] = game.handId.token()
         else:
             kwargs['token'] = None
@@ -358,6 +385,9 @@ class DeferredBlock(object):
 
     def tell(self, about, receivers, command, **kwargs):
         """send info about player 'about' to users 'receivers'"""
+        for kw in ('tile', 'tiles', 'meld', 'melds'):
+            if kw in kwargs:
+                kwargs[kw] = str(kwargs[kw]).encode()
         if about.__class__.__name__ == 'User':
             about = self.playerForUser(about)
         if not isinstance(receivers, list):
@@ -374,7 +404,7 @@ class DeferredBlock(object):
         for rec in self.__convertReceivers(receivers):
             isClient = rec.__class__.__name__.endswith('Client')
             if Debug.traffic and not isClient:
-                message = '-> {receiver:<15} about {about} {command}{kwargs}'.format(
+                message = u'-> {receiver:<15} about {about} {command}{kwargs}'.format(
                     receiver=rec.name[:15], about=about, command=command,
                     kwargs=Move.prettyKwargs(kwargs))
                 logDebug(message)
@@ -382,7 +412,12 @@ class DeferredBlock(object):
                 defer = Deferred()
                 defer.addCallback(rec.remote_move, command, **kwargs)
             else:
-                defer = self.table.server.callRemote(rec, 'move', aboutName, command.name, **kwargs)
+                defer = self.table.server.callRemote(
+                    rec,
+                    'move',
+                    aboutName,
+                    command.name,
+                    **kwargs)
             if defer:
                 defer.command = command.name
                 defer.notifying = 'notifying' in kwargs
@@ -393,7 +428,7 @@ class DeferredBlock(object):
             if isClient:
                 localDeferreds.append(defer)
         for defer in localDeferreds:
-            defer.callback(aboutName) # callback needs an argument !
+            defer.callback(aboutName)  # callback needs an argument !
 
     def tellPlayer(self, player, command, **kwargs):
         """address only one user"""
@@ -401,7 +436,12 @@ class DeferredBlock(object):
 
     def tellOthers(self, player, command, **kwargs):
         """tell others about player'"""
-        self.tell(player, list([x for x in self.table.game.players if x.name != player.name]), command, **kwargs)
+        self.tell(
+            player,
+            list(
+                [x for x in self.table.game.players if x.name != player.name]),
+            command,
+            **kwargs)
 
     def tellAll(self, player, command, **kwargs):
         """tell something to all players"""
