@@ -21,11 +21,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 # pylint: disable=unused-import, unused-wildcard-import, wildcard-import
 # pylint: disable=invalid-name
 
-import sip, sys
+import sip
+import sys
 
 from common import isPython3, Internal
 
-usingQt4 = True # Default for now
+usingQt4 = True  # Default for now
 usingQt5 = False
 
 if '--qt5' in sys.argv:
@@ -34,14 +35,19 @@ if '--qt5' in sys.argv:
         usingQt5 = True
         usingQt4 = False
     except ImportError as exc:
-        Internal.logger.debug('Cannot import Qt5:{}, using Qt4 instead'.format(exc.message))
+        Internal.logger.debug('{who}: Cannot import Qt5:{msg}, using Qt4 instead'.format(
+            who='Server' if Internal.isServer else 'Client',
+            msg=exc.message))
         from qt4 import *
 else:
     from qt4 import *
 
+
 class RealQVariant(object):
+
     """context helper, forcibly disabling QVariant autoconversion for Qt5.
     This makes it easier to write code supporting both Qt4 and Qt5"""
+
     def __init__(self):
         if usingQt5:
             sip.enableautoconversion(QVariant, False)
@@ -53,3 +59,10 @@ class RealQVariant(object):
         """enable autoconversion again"""
         if usingQt5:
             sip.enableautoconversion(QVariant, True)
+
+if isPython3:
+    def toQVariant(obj=None):
+        return obj
+else:
+    def toQVariant(obj=None):
+        return QVariant(obj)
