@@ -26,13 +26,17 @@ block that might have to be adapted.
 from subprocess import check_output, call
 from shutil import copy, move, copytree, rmtree
 
-import os, zipfile, tempfile
+import os
+import zipfile
+import tempfile
 
 # pylint:disable=invalid-name
 
+
 def makeIcon(svgName, icoName):
     """generates icoName.ico"""
     tmpDir = tempfile.mkdtemp(prefix='kaj')
+
     def pngName(resolution):
         """name for this resolution"""
         return '{}/kajongg{}.png'.format(
@@ -41,17 +45,25 @@ def makeIcon(svgName, icoName):
     try:
         for resolution in resolutions:
             pngFile = '{}{}.png'.format(icoName, resolution)
-            call('inkscape -z -e {outfile} -w {resolution} -h {resolution} {infile}'.format(
-                outfile=pngName(resolution), resolution=resolution, infile=svgName).split())
-        call('convert {pngFiles} {icoName}.ico'.format(pngFiles = ' '.join(pngName(x) for x in resolutions), icoName=icoName).split())
+            call(
+                'inkscape -z -e {outfile} -w {resolution} '
+                '-h {resolution} {infile}'.format(
+                    outfile=pngName(resolution),
+                    resolution=resolution,
+                    infile=svgName).split())
+        call('convert {pngFiles} {icoName}.ico'.format(
+            pngFiles=' '.join(pngName(x)
+                              for x in resolutions), icoName=icoName).split())
     finally:
         for resolution in resolutions:
             if os.path.exists(pngName(resolution)):
                 os.remove(pngName(resolution))
         os.rmdir(tmpDir)
 
-dataDir = check_output("kde4-config --expandvars --install data".format(type).split()).strip()
-iconDir = check_output("kde4-config --expandvars --install icon".format(type).split()).strip()
+dataDir = check_output(
+    "kde4-config --expandvars --install data".format(type).split()).strip()
+iconDir = check_output(
+    "kde4-config --expandvars --install icon".format(type).split()).strip()
 oxy48 = iconDir + '/oxygen/48x48'
 oxy48Cat = oxy48 + '/categories/'
 oxy48Act = oxy48 + '/actions/'
@@ -78,11 +90,14 @@ copy(oxy48Status + 'dialog-information.png', DEST + '/icons')
 copy(oxy48Status + 'dialog-warning.png', DEST + '/icons')
 copy(oxy48Apps + 'preferences-plugin-script.png', DEST + '/icons')
 
-for png in ('application-exit', 'games-config-background', 'arrow-right', 'format-list-ordered',
-            'object-rotate-left', 'help-contents', 'dialog-close', 'im-user', 'draw-freehand',
-            'call-start', 'configure', 'games-config-tiles', 'arrow-right-double', 'document-new',
-            'edit-delete', 'document-open', 'list-add-user', 'list-remove-user', 'configure-toolbars',
-            'go-up', 'go-down', 'go-next', 'go-previous'):
+for png in (
+        'application-exit', 'games-config-background', 'arrow-right',
+        'format-list-ordered', 'object-rotate-left', 'help-contents',
+        'dialog-close', 'im-user', 'draw-freehand', 'call-start',
+        'configure', 'games-config-tiles', 'arrow-right-double',
+        'document-new', 'edit-delete', 'document-open', 'list-add-user',
+        'list-remove-user', 'configure-toolbars',
+        'go-up', 'go-down', 'go-next', 'go-previous'):
     copy('{}/{}.png'.format(oxy48Act, png), DEST + '/icons')
 
 oggdec = 'oggdecV1.9.9.zip'
@@ -100,24 +115,38 @@ copy('tilesetselector.ui', DEST + '/kde4/apps/kajongg')
 copy('../hisc-apps-kajongg.svgz', DEST + '/icons/kajongg.svgz')
 
 makeIcon('../hisc-apps-kajongg.svgz', 'kajongg')
-makeIcon(iconDir + '/hicolor/scalable/actions/games-kajongg-law.svgz', 'games-kajongg-law')
+makeIcon(
+    iconDir +
+    '/hicolor/scalable/actions/games-kajongg-law.svgz',
+    'games-kajongg-law')
 copy('kajongg.ico', DEST + '/icons')
 copy('games-kajongg-law.ico', DEST + '/icons')
 
-# select sufficiently complete languages from http://l10n.kde.org/stats/gui/trunk-kde4/po/kajongg.po/
-languages = ('bs', 'ca', 'da', 'de', 'en_GB', 'es', 'et', 'fr', 'gl', 'it', 'kk', 'km', 'nl', 'nb', 'nds',
-   'pl', 'pt', 'pt_BR', 'ru', 'sl', 'sv', 'uk', 'zh_TW')
+# select sufficiently complete languages from
+# http://l10n.kde.org/stats/gui/trunk-kde4/po/kajongg.po/
+languages = (
+    'bs', 'ca', 'da', 'de', 'en_GB', 'es', 'et', 'fr', 'gl', 'it', 'kk',
+    'km', 'nl', 'nb', 'nds',
+    'pl', 'pt', 'pt_BR', 'ru', 'sl', 'sv', 'uk', 'zh_TW')
 
-#languages = ('de', 'zh_TW')
+# languages = ('de', 'zh_TW')
 
 for lang in languages:
     print 'getting language', lang
     os.makedirs(DEST + '/locale/{}/LC_MESSAGES'.format(lang))
-    for directory, filename in (('kdegames', 'kajongg'), ('kdegames', 'libkmahjongg'),
+    for directory, filename in (
+        ('kdegames', 'kajongg'), ('kdegames', 'libkmahjongg'),
             ('kdelibs', 'kdelibs4'), ('qt', 'kdeqt')):
-        mo_data = check_output('svn cat svn://anonsvn.kde.org/home/kde/trunk/l10n-kde4/{}/messages/{}/{}.po'.format(
-            lang, directory, filename).split())
-        with open('x.po'.format(filename), 'w') as outfile:
+        mo_data = check_output(
+            'svn cat svn://anonsvn.kde.org/home/kde/'
+            'trunk/l10n-kde4/{}/messages/{}/{}.po'.format(
+                lang, directory, filename).split())
+        with open('x.po'.format(filename), 'wb') as outfile:
             outfile.write(mo_data)
-        call('msgfmt x.po -o {}/locale/{}/LC_MESSAGES/{}.mo'.format(DEST, lang, filename).split())
+        call(
+            'msgfmt x.po -o {}/locale/{}/LC_MESSAGES/{}.mo'.format(
+                DEST,
+                lang,
+                filename).split(
+            ))
         os.remove('x.po')
